
/***************************************************************
 *
 * On canvas init
 *
 ***************************************************************/

{
  Tk_Item *currentItem = NULL;
  Tk_Item *newCurrent = NULL;
  int closeEnough = 0;  // integer pixels instead of the original 
  XEvent pickEvent;
  pickEvent.type = LeaveNotify;
  pickEvent.xcrossing.x = 0;
  pickEvent.xcrossing.y = 0;
  int state = 0;
  /*
   * Information used to speed up searches by remembering the last item
   * created or found with an item id search.
   */
  CP_Item *hotItem = NULL;
  CP_Item *hotPrev = NULL;
  
  char *takeFocus = NULL;
  int flags = 0;
  
  Tk_State canvas_state =  = TK_STATE_NORMAL;
}


/*--------------------------------------------------------------
 * CanvasBindProc --
 *	This procedure is invoked by the Tk dispatcher to handle
 *	events associated with bindings on items.
 * Results:
 *	None.
 * Side effects:
 *	Depends on the command invoked as part of the binding
 *	(if there was any).
 *--------------------------------------------------------------
 */
static void CanvasPlusBindProc(ClientData clientData, XEvent *event)
{
  CanvasPlus *canvas = (CanvasPlus *) clientData;

  PROTECT(canvas);
  /* This code below keeps track of the current modifier state in
   * canvasPtr>state.  This information is used to defer repicks of
   * the current item while buttons are down.
   */
  if ((event->type == ButtonPress) || (event->type == ButtonRelease)) {
      int mask;
      switch (event->xbutton.button) {
      case Button1: mask = Button1Mask; break;
      case Button2: mask = Button2Mask; break;
      case Button3: mask = Button3Mask; break;
      case Button4: mask = Button4Mask; break;
      case Button5: mask = Button5Mask; break;
      default: mask = 0;
      }
      if (event->type == ButtonPress) {
	/* On a button press, first repick the current item using
	 * the button state before the event, then process the event.*/  
	canvas->state = event->xbutton.state;
	PickCurrentItem(canvas, event);
	canvas->state ^= mask;
	ProcessEvent(canvas, event);
      } else {
	/* Button release: first process the event, with the button
	 * still considered to be down.  Then repick the current
	 * item under the assumption that the button is no longer down. */      
	canvas->state = event->xbutton.state;
	ProcessEvent(canvas, event);
	event->xbutton.state ^= mask;
	canvas->state = event->xbutton.state;
	PickCurrentItem(canvas, event);
	event->xbutton.state ^= mask;
      }
  } else if ((event->type == EnterNotify) || (event->type == LeaveNotify)) {
    canvas->state = event->xcrossing.state;
    PickCurrentItem(canvas, event);
  } else if (event->type == MotionNotify) {
    canvas->state = event->xmotion.state;
    PickCurrentItem(canvas, event);
    ProcessEvent(canvas, event);
  } else {
    ProcessEvent(canvas, event);
  }

  UNPROTECT(canvas);
}

/*--------------------------------------------------------------
 * PickCurrentItem --
 *	Find the topmost item in a canvas that contains a given
 *	location and mark the the current item.  If the current
 *	item has changed, generate a fake exit event on the old
 *	current item, a fake enter event on the new current item
 *	item and force a redraw of the two items. Canvas items
 *      that are hidden or disabled are ignored.
 * Results:
 *	None.
 * Side effects:
 *	The current item for canvas may change.  If it does,
 *	then the commands associated with item entry and exit
 *	could do just about anything.  A binding script could
 *	delete the canvas, so callers should protect themselves
 *	with Tcl_Preserve and Tcl_Release.
 *--------------------------------------------------------------
 */
static void PickCurrentItem(CanvasPlus *canvas, XEvent *event)
{
  int x, y, buttonDown;
  CP_Item *prevItem;
  /* Check whether or not a button is down.  If so, we'll log entry
   * and exit into and out of the current item, but not entry into
   * any other item.  This implements a form of grabbing equivalent
   * to what the X server does for windows.
   */
  buttonDown = Button1Mask|Button2Mask|Button3Mask|Button4Mask|Button5Mask;
  buttonDown &= canvas->state;
  if (!buttonDown) canvas->flags &= ~LEFT_GRABBED_ITEM;
  /*
   * Save information about this event in the canvas.  The event in
   * the canvas is used for two purposes:
   */
  if (event != &canvas->pickEvent) {
    if ((event->type == MotionNotify) || (event->type == ButtonRelease)) {
      canvas->pickEvent.xcrossing.type = EnterNotify;
      canvas->pickEvent.xcrossing.serial = event->xmotion.serial;
      canvas->pickEvent.xcrossing.send_event = event->xmotion.send_event;
      canvas->pickEvent.xcrossing.display = event->xmotion.display;
      canvas->pickEvent.xcrossing.window = event->xmotion.window;
      canvas->pickEvent.xcrossing.root = event->xmotion.root;
      canvas->pickEvent.xcrossing.subwindow = None;
      canvas->pickEvent.xcrossing.time = event->xmotion.time;
      canvas->pickEvent.xcrossing.x = event->xmotion.x;
      canvas->pickEvent.xcrossing.y = event->xmotion.y;
      canvas->pickEvent.xcrossing.x_root = event->xmotion.x_root;
      canvas->pickEvent.xcrossing.y_root = event->xmotion.y_root;
      canvas->pickEvent.xcrossing.mode = NotifyNormal;
      canvas->pickEvent.xcrossing.detail = NotifyNonlinear;
      canvas->pickEvent.xcrossing.same_screen = event->xmotion.same_screen;
      canvas->pickEvent.xcrossing.focus = False;
      canvas->pickEvent.xcrossing.state = event->xmotion.state;
    } else  {
      canvas->pickEvent = *event;
    }
  }
  /* If this is a reentrant call (there's already a partially completed
   * call pending on the stack;  it's in the middle of processing a
   * Leave event handler for the old current item) then just return;
   * the pending call will do everything that's needed.
   */
  if (canvas->flags & REPICK_IN_PROGRESS) return;
  /* A LeaveNotify event automatically means that there's no current
   * object, so the check for closest item can be skipped.
   */
  x = canvas->pickEvent.xcrossing.x + canvas->xOrigin;
  y = canvas->pickEvent.xcrossing.y + canvas->yOrigin;

  if (canvas->pickEvent.type != LeaveNotify)
    canvas->newCurrent = CPClosestItem(canvas, x, y);
  else
    canvas->newCurrent = NULL;

  /* Nothing to do:  the current item hasn't changed. */  
  if ((canvas->newCurrent==canvas->currentItem) && !(canvas->flags & LEFT_GRABBED_ITEM))
    return;

  /* Simulate a LeaveNotify event on the previous current item and
   * an EnterNotify event on the new current item.  Remove the "current"
   * tag from the previous current item and place it on the new current
   * item.
   */
  if ((canvas->newCurrent != canvas->currentItem) && 
      (canvas->currentItem != NULL) && !(canvas->flags & LEFT_GRABBED_ITEM)) {
    XEvent leave;
    CP_Item *item = canvas->currentItem;
    int i;

    leave = canvas->pickEvent;
    leave.type = LeaveNotify;
    leave.xcrossing.detail = NotifyAncestor;
    canvas->flags |= REPICK_IN_PROGRESS;
    ProcessEvent(canvas, &leave);
    canvas->flags &= ~REPICK_IN_PROGRESS;
    /* The check below is needed because there could be an event
     * handler for <LeaveNotify> that deletes the current item. */
    if ((item == canvas->currentItem) && !buttonDown) {
      Tk_Uid currentTag = CPGetCurrentTagUid();
      for (i = item->tags.number-1; i >= 0; i--) {
	if (item->tags.array[i] == currentTag) {
	  item->tags.array[i] = item->tags.array[item->tags.number-1];
	  item->tags.number--;
	  break;
	}
      }
    }
  }
  if ((canvas->newCurrent != canvas->currentItem) && buttonDown) {
    canvas->flags |= LEFT_GRABBED_ITEM;
    return;
  }

  /* Special note:  it's possible that canvas->newCurrent ==
   * canvas->currentItem here.  This can happen, for example,
   * if LEFT_GRABBED_ITEM was set. */
  
  prevItem = canvas->currentItem;
  canvas->flags &= ~LEFT_GRABBED_ITEM;
  canvas->currentItem = canvas->newCurrent;

  if (prevItem != NULL && prevItem != canvas->currentItem &&
      (prevItem->redraw_flags & TK_ITEM_STATE_DEPENDANT))
    EventuallyRedrawItem((CanvasPlus *)canvas, prevItem);
  
  if (canvas->currentItem != NULL)
    {
      XEvent enter;
      CPAddBindTag(canvas->currentItem, CPGetCurrentTagUid());
      if ((canvas->currentItem->redraw_flags & TK_ITEM_STATE_DEPENDANT &&
	   prevItem != canvas->currentItem)) {
	EventuallyRedrawItem((CanvasPlus *)canvas, canvas->currentItem);
      }
      enter = canvas->pickEvent;
      enter.type = EnterNotify;
      enter.xcrossing.detail = NotifyAncestor;
      ProcessEvent(canvas, &enter);
    }
}


/*--------------------------------------------------------------
 * ProcessEvent --
 *	This procedure is called to invoke binding processing
 *	for a new event that is associated with the current item
 *	for a canvas.
 * Results:
 *	None.
 * Side effects:
 *	Depends on the bindings for the canvas.  A binding script
 *	could delete the canvas, so callers should protect themselves
 *	with Tcl_Preserve and Tcl_Release.
 *--------------------------------------------------------------
 */
#define NUM_STATIC 5

static void ProcessEvent(CanvasPlus *canvas, XEvent *event)
{
  ClientData staticObjects[NUM_STATIC];
  ClientData *obj;
  int nObj, i;
  CP_Item *item;

  if (canvas->bindingTable == NULL) return;

  if ((event->type == KeyPress) || (event->type == KeyRelease))
    item = canvas->textInfo.focusItem;
  else
    item = canvas->currentItem;

  if (item == NULL)  return;

  nObj = item->tags.number + 2;

  if (nObj <= NUM_STATIC)
    obj = staticObjects;
  else
    obj = (ClientData *)ckalloc((unsigned)(nObj * sizeof(ClientData)));

  obj[0] = (ClientData) CPGetAllTagUid();        // Tag for "all" must be always present

  for (i = item->tags.number-1; i >= 0; i--)     // pick all tags uid for this item
    obj[i+1] = (ClientData) item->tags.array[i];

  obj[item->tags.number+1] = (ClientData) item;  // include item ptr in the objects list
  
  if (canvas->tkwin != NULL)
    Tk_BindEvent(canvas->bindingTable, event, canvas->tkwin, nObj, obj);

  if (obj != staticObjects) ckfree((char *) obj);
}

