/*
 ************************************************************************
 *
 * File 'modeTest.cpp'.
 *
 * CanvasPlus widget test mode implementation.
 *
 * Written by Jorge Lima in Nov 23 14:44:03 WET 2004
 *
 */
#include <tcl.h>
#include "tkCanvasPlusTags.h"
#include "tkCanvasPlus.h"
#include "tkCanvasPlusDrawing.h"
#include "tkCanvasPlusModes.h"

#include <iostream.h>
class iData;

class Item {
public:
  Item(iData *data, CP_Item *item);
  ~Item()
  {
  }
  Item *next()
  {
    return _next;
  }
  Item *prev()
  {
    return _prev;
  }
  virtual void draw(CP_DrawOps *ops, CP_Area *a);
  CP_Item *cp_item;
private:
  Item *_prev, *_next;
  iData *_data;
};

Item::Item(iData *data, CP_Item *item)
{ 
  cp_item = item;
  cp_item->clientData = (ClientData)this;
  _prev = 0;
  _next = 0;
  _data = data;
}

class PolylineItem: public Item {
public: 
  PolylineItem(iData *data, CP_Item *item);
  void draw(CP_DrawOps *ops, CP_Area *a);
private:
  CP_Point points[5];
};

class iData {
public:
  iData(CanvasPlus *_canvas)
  {
    canvas = _canvas;
    items = 0;
    n = 0;
  }
  ~iData() 
  {
    Item *item = items;
    Item *prev;
    while(item != 0) {
      prev = item;
      item = item->next();
      delete prev;
    }
  }
  CanvasPlus *getCanvas()
  { return canvas; }

  void append(Item *item)
  {
    n++;
    // item->next = items;
    items = item;
  }
  Item *getItems()
  {
    return items;
  }
  int count()
  {
    return n;
  }
private:
  CanvasPlus *canvas;
  int n;
  Item *items;
};

PolylineItem::PolylineItem(iData *data, CP_Item *item)
 : Item(data, item)
{
  points[0].x = 5; points[0].y = -10;
  points[1].x = 35; points[1].y = 60;
  points[2].x = 65; points[2].y = 10;
  points[3].x = 95; points[3].y = 60;
  points[4].x = 125; points[4].y = 10;
  CP_Area area;
  area.x1 = 5; area.y1 = -10;
  area.x2 = 125; area.y2 = 10;
  CPExpandPhysicalDimensions(data->getCanvas(), &area);
}

void PolylineItem::draw(CP_DrawOps *ops, CP_Area *a)
{
  ops->setPenColor(ops->clientData, 0);
  ops->setPenWidth(ops->clientData, 2.5);
  ops->drawPolyline(ops->clientData, 5, points);
}

/* FREE PROC */
static void freeProc(ClientData cd)
{
  iData *data = (iData *)cd;
  CanvasPlus *canvas = data->getCanvas();
  CPDeleteAllItems(canvas);
  delete data;
}

static ClientData allocProc(CanvasPlus *canvas)
{
  iData *data = new iData(canvas);
  return data;
}

static void drawProc(ClientData cd, CP_DrawOps *ops, CP_Area *a)
{
  iData *d = (iData *)cd;
  unsigned int addr = (unsigned int)d;
  cout << "OBJ PTR IS" << addr << endl;
  Item *item = d->getItems();
  while (item != NULL) {
    item->draw(ops, a);
    item = item->next();
  }
}

static CP_Item *closestProc(ClientData cd, CP_Point *pt, double dist)
{
   iData *d = (iData *)cd;
   return d->getItems()->cp_item;
}

 
/* PROCS TO REGISTER */
static int putsProc( ClientData clientData,
		     Tcl_Interp *interp,
		     int objc,
		     Tcl_Obj *CONST objv[])
{
  for(int i = 1; i < objc; i++)
    cout << Tcl_GetStringFromObj(objv[i], NULL) << " ";
  cout << endl;
  return TCL_OK;
}

static int polylineProc( ClientData clientData,
			 Tcl_Interp *interp,
			 int objc,
			 Tcl_Obj *CONST objv[])
{
  CP_Item *item;
  iData *data;
  CanvasPlus *canvas;
  // This returns a pointer to the mode specific data initialized
  // by allocProc() when the mode was set by CPSetMode()
  data = (iData *)clientData;
  canvas = data->getCanvas();
  cout << data->count() << endl;
  // Create a new Item. No private data was requested
  // must be initializer therafter
  item = CPCreateItem(canvas, 0);
  if (item==NULL) return TCL_ERROR;
  PolylineItem *polyline = new PolylineItem(data, item);
  data->append(polyline);
  return TCL_OK;
}

static int moveProc( ClientData clientData,
		     Tcl_Interp *interp,
		     int objc,
		     Tcl_Obj *CONST objv[])
{
  CP_Item *item;
  return TCL_OK;
}


/* REGISTER INFO */
static CP_RegisterInfo _info[] = {
  CP_ENTRY("@puts", putsProc),
  CP_ENTRY("@polyline", polylineProc),
  CP_ENTRY("@move", polylineProc),
  CP_LAST_ENTRY
};

static CP_ModeOps _operations = {
  allocProc,
  freeProc,
  drawProc,
  closestProc
};

/*
 * INIT must be declared as extern "C" if compiling in c++
 */

extern "C" int Modetest_Init(Tcl_Interp *interp)
{
  if ( CPDefineMode("testMode", _info, &_operations)!=TCL_OK )
    return TCL_ERROR;

  return TCL_OK;
}
