/*
 * ***********************************************************************
 *
 * File 'tkCanvasPlus.h'.
 *
 * CanvasPlus widget interface.
 *
 * Written by Jorge Lima in Nov 23 14:44:03 WET 2004
 *
 */

#ifndef _TK_CANVAS_PLUS_H
#define _TK_CANVAS_PLUS_H

#include <X11/Xutil.h>
#include <tcl.h>
#include <tk.h>
#include "CanvasPlusDefs.h"

#ifdef __cplusplus
extern "C" {
#endif

#define MAX_LAYERS  5
#define PROTECT(d)    Tcl_Preserve((ClientData)d)
#define UNPROTECT(d)  Tcl_Release((ClientData)d)
/*
 * Macros to test options
 */
#define BIT(n)            (0x01<<(n))

/*---------------------------------------------------------------
 *
 * Widget struct
 *
 *-------------------------------------------------------------*/

/* COORDINATES:
 *
 *  Physical coordinates - coordinates in physical units used in 
 *                         the high level abstration for the drawing 
 *                         page
 *
 *  Canvas coordinates - coordinates in screen dimensions but related
 *                       to the canvas true coordinate system.
 *
 *  Window coordinates - coordinates in screen dimensions relative to 
 *                       the window (i.e the viewing area) where the
 *                       The canvas is being displayed. In this
 *                       coordinate system the origin in the upper
 *                       left corner of the window.
 *
 *  Screen coordinates - same as Window coordinates but the origin
 *                       is located in the upper left corner of the
 *                       screen.
 */

struct CanvasPlus {
  Tk_Window tkwin;	   /* Window that embodies the widget. NULL 
			    * means window has been deleted but
			    * widget record hasn't been cleaned up yet. */
  Tcl_Interp *interp;	   /* Interpreter associated with widget. */
  Tcl_Command widgetCmd;   /* Token for widget command. */

  /* Token representing the configuration specifications. */
  Tk_OptionTable optionTable; 
  /* Option mask. Used by TK_SetOptions to indicate which options were set */
  int optMask;
  /* Fields to store options */
  int width, height;     /* widget dimensions */
  double zoom;           /* zoom factor of the viewing window relative to the canvas */
  int confine;           /* confine the canvas to show the maximum in the view area */
  int closeEnough;       /* integer pixels instead of the original double */
  int yInvert;           /* Invert the y axis */
  char *takeFocus;
  /* END of options */

  /* variables to hold the commands for scrollbar interaction */
  char *xScrollCmd, *yScrollCmd;
  /* limits of the canvas in canvas coordinates */
  int scrollX1, scrollY1, scrollX2, scrollY2;
  /* drawing limits in physical dimensions */
  double Xmin, Ymin, Xmax, Ymax;
  /* origin of the viewing window in canvas coordinates */
  int xOrigin, yOrigin;
  /* area that must be redrawn, in window coordinates */
  int redrawX1, redrawX2, redrawY1, redrawY2;
  /* Screen scale correction */
  double x_scale, y_scale, l_scale;
  /* Shared Graphics context */
  GC copyGC;
  Drawable drawable;
  int drawableMargin;
  int drawableXOrigin, drawableYOrigin;
  /* struct pointer for mode */
  CP_Mode *mode;

  /* Items */
  CP_Item *currentItem;
  CP_Item *newCurrent;
  XEvent pickEvent;
  int state;
  int flags;
  // see tkCanvas.c howto implement this property
  //Tk_State canvas_state;
};

/*
 * Macros for the flags field
 */
#define REDRAW_PENDING		1
#define REDRAW_BORDERS		2
#define REPICK_NEEDED		4
#define GOT_FOCUS		8
#define CURSOR_ON		0x10
#define UPDATE_SCROLLBARS	0x20
#define LEFT_GRABBED_ITEM	0x40
#define REPICK_IN_PROGRESS	0x100
#define BBOX_NOT_EMPTY		0x200


void CPForceRedraw(CanvasPlus *cPtr);

#ifdef __cplusplus
}
#endif


#endif /* _TK_CANVAS_PLUS_H */
