/*
 * ***********************************************************************
 *
 * File 'tkCanvasPlusDrawing.h'.
 *
 * CanvasPlus drawing rotines interface file.
 *
 * Written by Jorge Lima in Nov 23 14:44:03 WET 2004
 *
 */

#ifndef _TK_CANVAS_PLUS_DWG_H
#define _TK_CANVAS_PLUS_DWG_H

#include "CanvasPlusDefs.h"
#include "tkCanvasPlusTags.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Physical point
 */
struct CP_Point {
  double x, y;
};

/*
 * Physical area
 */
struct CP_Area {
  double x1, y1, x2, y2;
};

#define CP_AREA_DISJOINT     0
#define CP_AREA_INTERSECT    1
#define CP_AREA_CONTAINS     2
#define CP_AREA_ISCONTAINED  3

/*
 * Drawing operations
 */

typedef int CP_SelectPen(ClientData, int);
typedef int CP_SetPenColor(ClientData, char *, char *);
typedef void CP_SetPenWidth(ClientData, double);
typedef void CP_SetFont(ClientData, char *);
typedef void CP_SetFontSize(ClientData, int);
typedef void CP_DrawPoints(ClientData, int, CP_Point *);
typedef void CP_DrawPolyline(ClientData, int, CP_Point *);
typedef void CP_DrawPolygon(ClientData, int, CP_Point *, int, int);
typedef void CP_DrawRectangle(ClientData, CP_Area *, int);
typedef void CP_DrawElipse(ClientData, CP_Area *, int);
typedef void CP_DrawText(ClientData, double, double, char *);


struct CP_DrawOps {
  // this point to canvas if drawing on canvas
  ClientData clientData;
  // Pen configuration
  CP_SelectPen *selectPen;
  CP_SetPenColor *setPenColor; 
  CP_SetPenWidth *setPenWidth;
  // Font selection for the current pen
  CP_SetFont *setFont;
  CP_SetFontSize *setFontSize;
  // Drawing operations
  CP_DrawPoints *drawPoints;
  CP_DrawPolyline *drawPolyline;
  CP_DrawPolygon *drawPolygon;
  CP_DrawRectangle *drawRectangle;
  CP_DrawElipse *drawElipse;
  CP_DrawText *drawText;
};


/*
 * Item structure
 */
struct CP_Item {
  int id;
  CP_Tags tags;
  CP_Item *next;
  CP_Item *prev;
  ClientData clientData;
};

#define CP_STATIC_PEN_SET  10

struct CP_Pen {
  GC gc;
  XColor *fcolor;
  XColor *bcolor;
  Tk_Font font;
};

struct CP_PenSet {
  CP_Pen penStatic[CP_STATIC_PEN_SET];
  CP_Pen *penArray;
  CP_Pen *currentPen;
  int penCount;
  int penArraySize;
};



/*
 * Units for drawing
 */
#define CP_UNIT_MIL    0.0254
#define CP_UNIT_MM     1.0
#define CP_UNIT_CM    10.0
#define CP_UNIT_IN    25.4
#define CP_UNIT_MTR 1000.0

/* Configuration rotine */
void CPSetUnit(CanvasPlus *cp, double unit);
void CPRequestPhysicalDimensions(CanvasPlus *cPtr, const CP_Area *area);  
void CPExpandPhysicalDimensions(CanvasPlus *cPtr, const CP_Area *area);
void CPRecomputeCanvasDimensions(CanvasPlus *cPtr);
void CPExportEventInfo(CanvasPlus *cp, XEvent *event);

/* Item management rotines */
CP_Item *CPCreateItem(CanvasPlus *cp, size_t size);
void CPDeleteItem(CanvasPlus *cp, CP_Item *cpi);
void CPDeleteAllItems(CanvasPlus *cp);
Tcl_Obj *CPListAllItems(CP_Mode *mode);
Tcl_Obj *CPListTaggedItems(CP_Mode *mode, Tk_Uid tagUid);
CP_Item *CPClosestItem(CanvasPlus *cp, int x, int y);
CP_Item *CPGetItemFromId(CP_Mode *mode, int id);


/*********************************************************************
 *
 * Draw engine initialization
 *
 *********************************************************************/
void CPInitPenSet(CP_Mode *);
int CPCreatePen(CP_Mode *,  XGCValues *, unsigned long);
void CPDeletePenSet(CP_Mode *);

void CPDrawContents(CanvasPlus *);


#ifdef __cplusplus
}
#endif

#endif /* _TK_CANVAS_PLUS_DWG_H */
