
/************************** UTILS ************************************

*********************************************************************/

#include <tcl.h>


/*----------------------------------------------------------------------
 * GetStringsFromObjs
 *
 * Results:
 *	Converts object list into string list.
 * Side effects:
 *	Memory is allocated for the argv array, which must
 *	be freed using ckfree() when no longer needed.
 *---------------------------------------------------------------------*/
static CONST char **GetStringsFromObjs(int argc, Tcl_Obj *CONST objv[])
{
    register int i;
    CONST char **argv;
    if (argc <= 0) {
	return NULL;
    }
    argv = (CONST char **) ckalloc((argc+1) * sizeof(char *));
    for (i = 0; i < argc; i++) {
	argv[i]=Tcl_GetStringFromObj(objv[i], (int *) NULL);
    }
    argv[argc] = 0;
    return argv;
}

/*----------------------------------------------------------------------
 * ScrollFractions --
 *	Given the range that's visible in the window and the "100%
 *	range" for what's in the canvas, return a list of two
 *	doubles representing the scroll fractions.  This procedure
 *	is used for both x and y scrolling.
 * Results:
 *	The memory pointed to by string is modified to hold
 *	two real numbers containing the scroll fractions (between
 *	0 and 1) corresponding to the other arguments.
 * Side effects:
 *	None.
 *---------------------------------------------------------------------*/
static Tcl_Obj *
ScrollFractions(int screen1, int screen2, int object1, int object2)
{
    double range, f1, f2;
    char buffer[2*TCL_DOUBLE_SPACE+2];

    range = object2 - object1;
    if (range <= 0) {
	f1 = 0;	f2 = 1.0;
    } else {
	f1 = (screen1 - object1)/range;
	if (f1 < 0) f1 = 0.0;
	f2 = (screen2 - object1)/range;
	if (f2 > 1.0) f2 = 1.0;
	if (f2 < f1) f2 = f1;
    }
    sprintf(buffer, "%g %g", f1, f2);
    return Tcl_NewStringObj(buffer, -1);
}
